/*
 * GebuehrHinzuPruefer.java
 * eu.gronos.kostenrechner.controller.gebuehren (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.Component;

import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JList;

import eu.gronos.kostenrechner.controller.NumberExtractor;
import eu.gronos.kostenrechner.controller.system.FehlerHelper;
import eu.gronos.kostenrechner.interfaces.ParsendUndBauend;
import eu.gronos.kostenrechner.model.gebuehren.AuslagenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenAnrechnungsTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenErhoehungsTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenRahmenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.MehrfachPauschalTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.PauschalTatbestand;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;

/**
 * Der Prüfer kontrolliert alle Werte zum Bau eines {@link GebuehrenTatbestand}s
 *
 * @author Peter Schuster (setrok)
 * @date 26.06.2019
 *
 */
public class GebuehrenPruefer implements ParsendUndBauend<GebuehrenTatbestand> {
	private JFormattedTextField ftfAuslagenHoehe;
	private JLabel lblAuslagenHoehe;
	private JList<GebuehrenTatbestand> liGebuehrenListe;
	private GebuehrenTatbestand rueckgabewert;

	/**
	 * @param components braucht ein {@link JFormattedTextField}, das dazu gehörende
	 *                   {@link JLabel} und eine {@link JList}.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.ParsendUndBauend#leseComponents(Component[])
	 */
	@SuppressWarnings("unchecked")
	@Override
	public ParsendUndBauend<GebuehrenTatbestand> leseComponents(Component... components) {
		if (components != null) {
			for (Component comp : components) {
				JComponent component = (JComponent) comp;
				if ("16035".equals(component.getName())) {
					ftfAuslagenHoehe = (JFormattedTextField) component;
				} else if ("16021".equals(component.getName())) {
					liGebuehrenListe = (JList<GebuehrenTatbestand>) component;
				} else if ("16031".equals(component.getName())) {
					lblAuslagenHoehe = (JLabel) component;
				}
			}
		}
		return this;
	}

	/**
	 * Die Methode überprüft den {@link #getRueckgabewert()}, nachdem er Felder
	 * ausgelesen hat.
	 * 
	 * @return true, wenn die Plausibilitäten zufrieden sind
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.ParsendUndBauend#parseEingabe()
	 */
	@Override
	public boolean parseEingabe() {
		try {
			rueckgabewert = leseFelder();
			parseEingabe(rueckgabewert);
		} catch (NumberFormatException | NullPointerException ne) {
			FehlerHelper.zeigeFehler(ne.getLocalizedMessage(), ne);
			return false;
		}
		return true;
	}

	/**
	 * Die Methode überprüft den {@link #getRueckgabewert()}, ohne Felder
	 * auszulesen, indem er ihr übergeben wird.
	 * 
	 * @param rueckgabewert der zu prüfende {@link GebuehrenTatbestand}
	 * @return true, wenn die Plausibilitäten zufrieden sind
	 * @throws NumberFormatException wenn der {@link NumberExtractor} bei einem
	 *                               {@link JFormattedTextField} eine solche
	 *                               Exception wirft oder Plausibilitäten nicht
	 *                               eingehalten sind.
	 * @throws NullPointerException  wenn kein {@link GebuehrenTatbestand} in der
	 *                               {@link JList} ausgewählt ist.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.ParsendUndBauend#parseEingabe(java.lang.Object)
	 */
	@Override
	public boolean parseEingabe(GebuehrenTatbestand gt) throws NumberFormatException, NullPointerException {
		this.rueckgabewert = gt;

		if (rueckgabewert instanceof MehrfachPauschalTatbestand) {
			// TODO: gefährlich, wenn beim XML-Parsen geprüft wird!
			if (((MehrfachPauschalTatbestand) rueckgabewert).getAnzahl() < 0)
				throw new NumberFormatException(HinzufuegenDialog.FEHLER_ANZAHL_NEGATIV);
		} else if (rueckgabewert instanceof AuslagenTatbestand) {
			// TODO: gefährlich, wenn beim XML-Parsen geprüft wird!
			if (((AuslagenTatbestand) rueckgabewert).getBetrag() < 0)
				throw new NumberFormatException(HinzufuegenDialog.FEHLER_AUSLAGENHOEHE_NEGATIV);
		} else if (rueckgabewert instanceof PauschalTatbestand) {
			// stand vorher in GebuehrenSatzTatbestand#setSatz()
			if (((PauschalTatbestand) rueckgabewert).getBetrag() < 0) {
				throw new NumberFormatException(
						String.format("Bei einer %s darf der Betrag nicht negativ sein, hier %,.2f",
								PauschalTatbestand.ART, ((PauschalTatbestand) rueckgabewert).getBetrag()));
			}
		} else if (rueckgabewert instanceof GebuehrenErhoehungsTatbestand) {
			if (((GebuehrenErhoehungsTatbestand) rueckgabewert).getAnzahl() < 0)
				throw new NumberFormatException(HinzufuegenDialog.FEHLER_ANZAHL_NEGATIV);
		} else if (rueckgabewert instanceof GebuehrenAnrechnungsTatbestand) {
			GebuehrenAnrechnungsTatbestand grt = (GebuehrenAnrechnungsTatbestand) rueckgabewert;
			if (Math.abs(grt.getSatz()) < Math.abs(grt.getUntergrenze())
					|| Math.abs(grt.getSatz()) > Math.abs(grt.getObergrenze())) {
				throw new NumberFormatException(
						String.format("Die angemessene Gebühr muss zwischen %,.2f- und %,.2f-fach liegen!",
								grt.getUntergrenze(), grt.getObergrenze()));
			}
		} else if (rueckgabewert instanceof GebuehrenRahmenTatbestand) {
			GebuehrenRahmenTatbestand grt = (GebuehrenRahmenTatbestand) rueckgabewert;
			if (grt.getSatz() < grt.getUntergrenze() || grt.getSatz() > grt.getObergrenze()) {
				// Damit wird auch geprüft, ob der Satz negativ ist
				throw new NumberFormatException(
						String.format("Die angemessene Gebühr muss zwischen %,.1f- und %,.1f-fach liegen!",
								grt.getUntergrenze(), grt.getObergrenze()));
			}
		} else if (rueckgabewert instanceof GebuehrenSatzTatbestand) {
			// Diese Prüfung stand vorher in GebuehrenSatzTatbestand#setSatz()
			if (((GebuehrenSatzTatbestand) rueckgabewert).getSatz() < 0) {
				throw new NumberFormatException(
						String.format("Bei einem %s darf der Satz nicht negativ sein, hier %,.2f",
								GebuehrenSatzTatbestand.ART, ((GebuehrenSatzTatbestand) rueckgabewert).getSatz()));
			}
		}

		return true;
	}

	/**
	 * @return einen {@link GebuehrenTatbestand}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.ParsendUndBauend#getRueckgabewert()
	 */
	@Override
	public GebuehrenTatbestand getRueckgabewert() {
		return rueckgabewert;
	}

	/**
	 * Die Methode gibt das ausgewählte Element der {@link JList} zurück.
	 * 
	 * @return {@link GebuehrenTatbestand}
	 */
	public GebuehrenTatbestand getSelectedItem() {
		if (liGebuehrenListe.getSelectedIndex() < 0)
			return null;
		else
			return (GebuehrenTatbestand) liGebuehrenListe.getModel().getElementAt(liGebuehrenListe.getSelectedIndex());
	}

	/**
	 * Übergibt den ausgewählten {@link GebuehrenTatbestand} bzw. baut aus dem
	 * ausgewählten {@link GebuehrenTatbestand} und den übrigen Werten einen
	 * {@link GebuehrenTatbestand}.
	 * 
	 * Bei {@link AuslagenTatbestand}, {@link GebuehrenRahmenTatbestand},
	 * {@link GebuehrenAnrechnungsTatbestand} und
	 * {@link GebuehrenErhoehungsTatbestand} wird geklont, um nicht die Werte im
	 * Original zu ändern.
	 * 
	 * @return einen {@link GebuehrenTatbestand}
	 * @throws NumberFormatException wenn der {@link NumberExtractor} bei einem
	 *                               {@link JFormattedTextField} eine solche
	 *                               Exception wirft.
	 * @throws NullPointerException  wenn kein {@link GebuehrenTatbestand} in der
	 *                               {@link JList} ausgewählt ist.
	 */
	private GebuehrenTatbestand leseFelder() throws NumberFormatException, NullPointerException {
		rueckgabewert = getSelectedItem();
		final NumberExtractor extractor = new NumberExtractor(ftfAuslagenHoehe, lblAuslagenHoehe.getText());
		if (rueckgabewert == null) {
			throw new NullPointerException(
					"Kein Gebührentatbestand ausgewählt. Wählen Sie bitte einen Gebührentatbestand aus!");
		} else if (rueckgabewert instanceof MehrfachPauschalTatbestand
				&& ((MehrfachPauschalTatbestand) rueckgabewert).getAnzahl() < 0) {
			int anzahl = extractor.extractInteger();
			rueckgabewert = ((MehrfachPauschalTatbestand) rueckgabewert).clone();
			((MehrfachPauschalTatbestand) rueckgabewert).setAnzahl(anzahl);
		} else if (rueckgabewert instanceof AuslagenTatbestand
				&& ((AuslagenTatbestand) rueckgabewert).getBetrag() < 0) {
			double auslagen = extractor.extractDouble();
			rueckgabewert = ((AuslagenTatbestand) rueckgabewert).clone();
			((AuslagenTatbestand) rueckgabewert).setBetrag(auslagen);
		} else if (rueckgabewert instanceof PauschalTatbestand) {
			rueckgabewert = (PauschalTatbestand) rueckgabewert;
		} else if (rueckgabewert instanceof GebuehrenErhoehungsTatbestand
				&& ((GebuehrenErhoehungsTatbestand) rueckgabewert).getAnzahl() < 0) {
			int anzahl = extractor.extractInteger();
			rueckgabewert = ((GebuehrenErhoehungsTatbestand) rueckgabewert).clone();
			((GebuehrenErhoehungsTatbestand) rueckgabewert).setAnzahl(anzahl);
		} else if (rueckgabewert instanceof GebuehrenAnrechnungsTatbestand) {
			double satz = extractor.extractDouble();
			rueckgabewert = ((GebuehrenAnrechnungsTatbestand) rueckgabewert).clone();
			((GebuehrenAnrechnungsTatbestand) rueckgabewert).setSatz(satz);
		} else if (rueckgabewert instanceof GebuehrenRahmenTatbestand) {
			double satz = extractor.extractDouble();
			rueckgabewert = ((GebuehrenRahmenTatbestand) rueckgabewert).clone();
			((GebuehrenRahmenTatbestand) rueckgabewert).setSatz(satz);
		}
		return rueckgabewert;
	}
	//
	//

}

// instanceof JFormattedTextField
// instanceof JList<?>
// instanceof JLabel
//	GebuehrenTatbestand gt = getSelectedItem();
//	final NumberExtractor extractor = new NumberExtractor(ftfAuslagenHoehe, lblAuslagenHoehe.getText());
//	if (gt != null && gt instanceof AuslagenTatbestand && ((AuslagenTatbestand) gt).getBetrag() < 0) {
//		double auslagen = extractor.extractDouble();
//		gt = ((AuslagenTatbestand) gt).clone();
//		((AuslagenTatbestand) gt).setBetrag(auslagen);
//	} else if (gt != null && gt instanceof GebuehrenErhoehungsTatbestand
//			&& ((GebuehrenErhoehungsTatbestand) gt).getAnzahl() < 0) {
//		int anzahl = extractor.extractInteger();
//		gt = ((GebuehrenErhoehungsTatbestand) gt).clone();
//		((GebuehrenErhoehungsTatbestand) gt).setAnzahl(anzahl);
//	} else if (gt instanceof GebuehrenAnrechnungsTatbestand) {
//		double satz = extractor.extractDouble();
//		gt = ((GebuehrenAnrechnungsTatbestand) gt).clone();
//		((GebuehrenAnrechnungsTatbestand) gt).setSatz(satz);
//	} else if (gt instanceof GebuehrenRahmenTatbestand) {
//		double satz = extractor.extractDouble();
//		gt = ((GebuehrenRahmenTatbestand) gt).clone();
//		((GebuehrenRahmenTatbestand) gt).setSatz(satz);
//	}
//	return gt;

//	private boolean pruefe() {
//		GebuehrenTatbestand gt = getSelectedItem();
//		final NumberExtractor extractor = new NumberExtractor(ftfAuslagenHoehe, lblAuslagenHoehe.getText());
//		if (gt == null) {
//			try {
//				throw new NullPointerException(
//						"Kein Gebührentatbestand ausgewählt. Wählen Sie bitte einen Gebührentatbestand aus!");
//			} catch (NullPointerException npe) {
//				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, npe.getLocalizedMessage(), npe);
//				return false;
//			}
//		} else if (gt instanceof AuslagenTatbestand && ((AuslagenTatbestand) gt).getBetrag() < 0) {
//			try {
//				double auslagen = extractor.extractDouble();
//				if (auslagen < 0)
//					throw new NumberFormatException(HinzufuegenDialog.FEHLER_AUSLAGENHOEHE_NEGATIV);
//			} catch (NumberFormatException | NullPointerException ne) {
//				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ne.getLocalizedMessage(), ne);
//				return false;
//			}
//		} else if (gt instanceof GebuehrenErhoehungsTatbestand
//				&& ((GebuehrenErhoehungsTatbestand) gt).getAnzahl() < 0) {
//			try {
//				int anzahl = extractor.extractInteger();
//				if (anzahl < 0)
//					throw new NumberFormatException(HinzufuegenDialog.FEHLER_ANZAHL_NEGATIV);
//			} catch (NumberFormatException | NullPointerException ne) {
//				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ne.getLocalizedMessage(), ne);
//				return false;
//			}
//		} else if (gt instanceof GebuehrenAnrechnungsTatbestand) {
//			final GebuehrenAnrechnungsTatbestand grt = (GebuehrenAnrechnungsTatbestand) gt;
//			try {
//				double satz = extractor.extractDouble();
//				if (Math.abs(satz) < Math.abs(grt.getUntergrenze()) || Math.abs(satz) > Math.abs(grt.getObergrenze())) {
//					throw new NumberFormatException(
//							String.format("Die angemessene Gebühr muss zwischen %,.2f- und %,.2f-fach liegen!",
//									grt.getUntergrenze(), grt.getObergrenze()));
//				}
//			} catch (NumberFormatException | NullPointerException ne) {
//				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ne.getLocalizedMessage(), ne);
//				return false;
//			}
//		} else if (gt instanceof GebuehrenRahmenTatbestand) {
//			final GebuehrenRahmenTatbestand grt = (GebuehrenRahmenTatbestand) gt;
//			try {
//				double satz = extractor.extractDouble();
//				if (satz < grt.getUntergrenze() || satz > grt.getObergrenze()) {
//					throw new NumberFormatException(
//							String.format("Die angemessene Gebühr muss zwischen %,.1f- und %,.1f-fach liegen!",
//									grt.getUntergrenze(), grt.getObergrenze()));
//				}
//			} catch (NumberFormatException | NullPointerException ne) {
//				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ne.getLocalizedMessage(), ne);
//				return false;
//			}
//		}
//		return true;
//	}